# This file is part of the Hike distribution (https://github.com/hikelang or http://hikelang.github.io).
# Copyright (c) 2021 Salimgereyev Adi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import exceptions
from typing import Generic, TypeVar, List
from threading import Thread

T = TypeVar('T')


class AssignmentTestUnit(Generic[T]):
    def __init__(self, left: List[T], right: List[T]):
        self.left = left
        self.right = right

    def start(self):
        Thread(target=self.run).start()

    def run(self):
        if len(self.left) != len(self.right):
            raise exceptions.AssignmentTestDifferentValues(
                "Length of self.left: {} != length of self.right: {}".format(
                    self.left,
                    self.right))
        for index in range(len(self.left)):
            if self.left[index] != self.right[index]:
                raise exceptions.AssignmentTestDifferentValues(
                    "self.left[{1}] != self.right[{1}]: {2} != {3}".format(
                        index,
                        self.left[index],
                        self.right[index]
                    )
                )
        print("tests passed")



# This file is part of the Hike distribution (https://github.com/hikelang or http://hikelang.github.io).
# Copyright (c) 2021 Salimgereyev Adi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import language_codes
from distutils.spawn import find_executable


class Compiler:
    def __init__(self, name: str, compile_command_format: str, language_code: int):
        self.name = name
        self.compile_command_format = compile_command_format
        self.language_code = language_code

    def exists(self) -> bool:
        """ Checks if compiler is installed in OS.
        """
        return find_executable(self.name) is not None

    def __str__(self) -> str:
        return "ESTEST.Compiler(name=`{}`, compile_command_format=`{}`)".format(self.name,
                                                                                self.compile_command_format)


GCC = Compiler(
    "gcc", "gcc -fPIC -shared -o {2} {1}", language_codes.LANGUAGE_C)
GPP = Compiler(
    "g++", "g++ -fPIC -shared -o {2} {1}", language_codes.LANGUAGE_CPP)
CLANG = Compiler(
    "clang", "clang -shared -undefined dynamic_lookup -o {2} {1}", language_codes.LANGUAGE_CPP)



# This file is part of the Hike distribution (https://github.com/hikelang or http://hikelang.github.io).
# Copyright (c) 2021 Salimgereyev Adi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import compilers
import language_codes
import exceptions
import os
from typing import List


class SharedObjectBuilder:
    def __init__(self, compiler: compilers.Compiler, sources: List[str], output_shared_object: str):
        if not compiler.exists():
            raise exceptions.CompilerNotFound(
                "compiler `{}` was not found in your OS".format(compiler.name))
        self.compiler = compiler
        self.sources = sources
        self.output_shared_object = output_shared_object

    def detect_language(self) -> int:
        current_language_code: int = 0
        global_language_code: int = 0
        for source in self.sources:
            if source.endswith(".c"):
                current_language_code = language_codes.LANGUAGE_C
            elif source.endswith(".cpp"):
                current_language_code = language_codes.LANGUAGE_CPP
            else:
                raise exceptions.WrongSourceFound(
                    "ESTEST cannot handle language of {}".format(source))
            global_language_code = max(
                current_language_code, global_language_code)
        return global_language_code

    def build_sources(self) -> int:
        return os.system(self.compiler.compile_command_format.format(
            " ".join(self.sources),
            self.output_shared_object))

    def build(self) -> int:
        language = self.detect_language()
        if self.compiler.language_code < language:
            raise exceptions.LanguageNotSupported("The compiler is not suitable for compilation of {} language.".format(
                language_codes.language_code_to_string(language)))
        compilation_return_code: int = self.build_sources()
        if compilation_return_code != 0:
            raise exceptions.CompilationError(
                "Compilation failed with code: {}".format(compilation_return_code))

    def __str__(self) -> str:
        return "ESTEST.SharedObjectBuilder(compiler=`{}`, sources={})".format(self.compiler.name, self.sources)



# This file is part of the Hike distribution (https://github.com/hikelang or http://hikelang.github.io).
# Copyright (c) 2021 Salimgereyev Adi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '1.0.0'
DESCRIPTION = 'A simple testing framework'

# Setting up
setup(
    name="estest",
    version=VERSION,
    author="Salimgereyev Adi",
    author_email="<adisalimgereev@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['opencv-python', 'pyautogui', 'pyaudio'],
    keywords=['python', 'testing', 'c', 'c++'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



# This file is part of the Hike distribution (https://github.com/hikelang or http://hikelang.github.io).
# Copyright (c) 2021 Salimgereyev Adi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

""" Is language code is bigger, then the priority of the language is bigger.
"""
LANGUAGE_C = 1
LANGUAGE_CPP = 2


def language_code_to_string(language_code: int):
    if language_code == 1:
        return LANGUAGE_C
    elif language_code == 2:
        return LANGUAGE_CPP
